# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import time
import os

from qommon.ctl import Command

def clean_vhost_sessions():
    import sessions
    manager = sessions.StorageSessionManager()
    one_week_ago = time.time() - 2*86400
    one_month_ago = time.time() - 30*86400
    for session_key in manager.keys():
        try:
            session = manager.get(session_key)
        except AttributeError:
            del manager[session_key]
            continue
        if session._access_time < one_week_ago or session._creation_time < one_month_ago:
            del manager[session.id]


class CmdCleanSessions(Command):
    name = 'clean_sessions'

    def execute(self, base_options, sub_options, args):
        import publisher

        publisher.WcsPublisher.configure(self.config, sub_options.extra)
        pub = publisher.WcsPublisher.create_publisher()

        app_dir = pub.app_dir
        hostnames = os.listdir(app_dir)
        for hostname in hostnames:
            pub.app_dir = os.path.join(app_dir, hostname)
            clean_vhost_sessions()

CmdCleanSessions.register()
