# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import json

from qommon.form import *
from qommon.misc import http_get_page, http_post_request, get_variadic_url
from wcs.workflows import WorkflowStatusItem, register_item_class

TIMEOUT = 15

class WebserviceCallStatusItem(WorkflowStatusItem):
    description = N_('Webservice Call')
    key = 'webservice_call'
    support_substitution_variables = True

    url = None
    varname = None
    post = True

    def get_parameters(self):
        return ('url', 'post', 'varname')

    def add_parameters_widgets(self, form, parameters, prefix='', formdef=None):
        if 'url' in parameters:
            form.add(StringWidget, '%surl' % prefix,
                     title=_('URL'), value=self.url, size=80,
                     hint=_('Common substitution variables are available with the [variable] syntax.'))
        if 'post' in parameters:
            form.add(CheckboxWidget, '%spost' % prefix,
                    title=_('Post formdata (JSON)'),
                    value=self.post)
        if 'varname' in parameters:
            form.add(VarnameWidget, '%svarname' % prefix,
                     title=_('Variable Name'), value=self.varname)

    def perform(self, formdata):
        if not self.url:
            # misconfigured action
            return
        url = self.url
        if '[' in url:
            variables = get_publisher().substitutions.get_context_variables()
            url = get_variadic_url(url, variables)
        headers = {'Content-type': 'application/json',
                'Accept': 'application/json'}
        if self.post:
            formdata_as_json = formdata.export_to_json()
            response, status, data, auth_header = http_post_request(
                    url, formdata_as_json, headers=headers, timeout=TIMEOUT)
        else:
            response, status, data, auth_header = http_get_page(
                    url, headers=headers, timeout=TIMEOUT)
        if self.varname:
            workflow_data = {'%s_status' % self.varname: status}
            if status == 200:
                try:
                    d = json.loads(data)
                except (ValueError, TypeError):
                    pass
                else:
                    workflow_data['%s_response' % self.varname] = d
                    if isinstance(d.get('data'), dict) and d['data'].get('display_id'):
                        formdata.id_display = d.get('data', {}).get('display_id')
            formdata.update_workflow_data(workflow_data)
            formdata.store()

        if (status // 100) not in (2, 3):
            raise Exception("Call to webservice gave %s as status code" % status)

register_item_class(WebserviceCallStatusItem)
